#include <qpainter.h>
#include <qtextcodec.h>
#include "decodew.H"

DecodeWindow::DecodeWindow(RSAKey *kpriv,RSAEncodedMsg *m,QPixmap *bkg,QWidget *parent,char *name) : QWidget(parent,name){
  privkey=kpriv;
  bkg_pixmap=bkg;
  msg=m;
  
  QTextCodec *tc=QTextCodec::codecForName("iso8859-2");
  srt[srt_title]=new QSimpleRichText(tc->toUnicode("<h2><font color=darkblue>Deszyfrowanie kryptogramu RSA</font></h2>"),font());
  srt[srt_brakkluczy]=new QSimpleRichText(tc->toUnicode("<h2><font color=red>Najpierw wygeneruj klucze!</font></h2>"),font());
  srt[srt_brakmsg]=new QSimpleRichText(tc->toUnicode("<h2><font color=red>Najpierw co zaszyfuj!</font></h2>"),font());
  srt[srt_kpriv]=kpriv2srt();
  srt[srt_tekstjawny]=new QSimpleRichText(tc->toUnicode("co po zamianie ASCII na litery daje"),font());
  srt[srt_tekstascii]=new QSimpleRichText(tc->toUnicode("dekodujemy zgodnie z wzorem M=C^d mod n, gdzie C to zaszyfrowana litera"),font());
  srt[srt_tekstrsa]=new QSimpleRichText(tc->toUnicode("Zaszyfrowany tekst"),font());
  for (int i=0; i<nr_tv; i++) {
    tv[i]=new QTextView(this);
  }
  if ((privkey->d>=2)&&(privkey->n>=2)&&(msg->len>0)){
    for (int i=0; i<nr_tv; i++) tv[i]->show();
  } else {
    for (int i=0; i<nr_tv; i++) tv[i]->hide();
  }
  updateTV();
}

DecodeWindow::~DecodeWindow(){
  for (int i=0; i<nr_srt; i++) delete srt[i];
  for (int i=0; i<nr_tv; i++) delete tv[i];
}

void DecodeWindow::paintEvent(QPaintEvent *){
  QPixmap pm(size());
  QPainter p;
  p.begin(&pm);
  if (bkg_pixmap->isNull())
    pm.fill(backgroundColor());
  else 
    p.drawTiledPixmap(0,0,width(),height(),*bkg_pixmap);
  int cy=5;
  srt[srt_title]->draw(&p,width()/2-srt[srt_title]->widthUsed()/2,cy,QRegion(0,0,width(),height()),colorGroup());
  cy+=srt[srt_title]->height();
  if ((privkey->d<2)||(privkey->n<2)){
    cy+=srt[srt_title]->height();
    srt[srt_brakkluczy]->draw(&p,width()/2-srt[srt_brakkluczy]->widthUsed()/2,cy,QRegion(0,0,width(),height()),colorGroup());
    cy+=srt[srt_brakkluczy]->height();
}
  if ((privkey->d>=2)&&(privkey->n>=2)){
    srt[srt_kpriv]->draw(&p,5,cy,QRegion(0,0,width(),height()),colorGroup());
    cy+=srt[srt_kpriv]->height();
    if (msg->len<=0){
      srt[srt_brakmsg]->draw(&p,width()/2-srt[srt_brakmsg]->widthUsed()/2 ,cy,QRegion(0,0,width(),height()),colorGroup());
      cy+=srt[srt_brakmsg]->height();
    } else {
      srt[srt_tekstrsa]->draw(&p,5,cy,QRegion(0,0,width(),height()),colorGroup());
      srt[srt_tekstascii]->draw(&p,5,tv[tv_rsa]->y()+tv[tv_rsa]->height(),QRegion(0,0,width(),height()),colorGroup());
      srt[srt_tekstjawny]->draw(&p,5,tv[tv_ascii]->y()+tv[tv_ascii]->height(),QRegion(0,0,width(),height()),colorGroup());
    }
  }
  p.end();
  bitBlt(this,0,0,&pm);
}

void DecodeWindow::resizeEvent(QResizeEvent *){
  for (int i=0; i<nr_srt; i++) srt[i]->setWidth(width());
  tv[tv_rsa]->setGeometry(0,srt[srt_title]->height()+srt[srt_kpriv]->height()+srt[srt_tekstrsa]->height()+10,width(),height()/3);
  tv[tv_ascii]->setGeometry(0,tv[tv_rsa]->y()+tv[tv_rsa]->height()+srt[srt_tekstascii]->height()+5,width(),5*srt[srt_tekstrsa]->height());
  tv[tv_plain]->setGeometry(0,tv[tv_ascii]->y()+tv[tv_ascii]->height()+srt[srt_tekstjawny]->height()+5,width(),3*srt[srt_tekstrsa]->height());
  updateTV();
}

QSimpleRichText * DecodeWindow::kpriv2srt(){
  char ds[10240],ns[10240];
  mpz_get_str(ds,10,privkey->d.get_mpz_t());
  mpz_get_str(ns,10,privkey->n.get_mpz_t());
  return new QSimpleRichText(QString("<nobr>klucz prywatny <font color=blue>(d,n)=(")+QString(ds)+QString(",")+QString(ns)+QString(")</font></nobr>"),font());
}

void DecodeWindow::updateTV(){
  if (msg->len>0){
    tv[tv_rsa]->setText(QString(""));
    for (int i=0; i<msg->len; i++){
      char t1[10240];
      t1[0]=0;
      mpz_get_str(t1,10,msg->msg[i]);
      QString ptvr=tv[tv_rsa]->text();
      tv[tv_rsa]->setText(tv[tv_rsa]->text()+QString(t1)+QString(", "));
      if (tv[tv_rsa]->visibleWidth()<tv[tv_rsa]->contentsWidth()){
	tv[tv_rsa]->setText(ptvr+QString("<br>")+QString(t1)+QString(", "));
	
      }
    }
    char *t=new char[msg->len+1];
    t[msg->len]=0;
    msg->decode(t,*privkey);
    tv[tv_ascii]->setText("");
    for (unsigned int i=0; i<strlen(t); i++){
      QString si;
      si.setNum(int(t[i]));
      QString ptva=tv[tv_ascii]->text();
      tv[tv_ascii]->setText(tv[tv_ascii]->text()+si+QString(", "));
      if (tv[tv_ascii]->visibleWidth()<tv[tv_ascii]->contentsWidth()){
	tv[tv_ascii]->setText(ptva+QString("<br>")+si+QString(", "));
	
      }
    }
    tv[tv_plain]->setText(t);
    delete[] t;
  }
}

