#include <qpainter.h>
#include <qtextcodec.h>
#include "encodew.H"

QString EncodeWindow::plain_text;

EncodeWindow::EncodeWindow(RSAKey *kpub,RSAEncodedMsg *m,QPixmap *bkg,QWidget *parent,char *name) : QWidget(parent,name){
  pubkey=kpub;
  bkg_pixmap=bkg;
  msg=m;
  
  QTextCodec *tc=QTextCodec::codecForName("iso8859-2");
  srt[srt_title]=new QSimpleRichText(tc->toUnicode("<h2><font color=darkblue>Szyfrowanie przy pomocy RSA</font></h2>"),font());
  srt[srt_brakkluczy]=new QSimpleRichText(tc->toUnicode("<h2><font color=red>Najpierw wygeneruj klucze!</font></h2>"),font());
  srt[srt_kpub]=kpub2srt();
  srt[srt_tekstjawny]=new QSimpleRichText(tc->toUnicode("Wpisz tekst jawny:"),font());
  srt[srt_tekstascii]=new QSimpleRichText(tc->toUnicode("Tekst jawny w postaci kodw ASCII"),font());
  srt[srt_tekstrsa]=new QSimpleRichText(tc->toUnicode("Blok M tekstu jawnego (w naszym przypadku jest to kod ascii pojedynczego znaku) kodujemy stosujc wzr: C=M^e mod n"),font());
  mln=new QMultiLineEdit(this);
  mln->setText(plain_text);
  for (int i=0; i<nr_tv; i++) {
    tv[i]=new QTextView(this);
  }
  if ((pubkey->e>=2)&&(pubkey->n>=2)){
    mln->show();
    for (int i=0; i<nr_tv; i++) tv[i]->show();
  } else {
    mln->hide();
    for (int i=0; i<nr_tv; i++) tv[i]->hide();
  }
  connect(mln,SIGNAL(textChanged()),this,SLOT(updateTV()));
}

EncodeWindow::~EncodeWindow(){
  for (int i=0; i<nr_srt; i++) delete srt[i];
  delete mln;
  for (int i=0; i<nr_tv; i++) delete tv[i];
}

void EncodeWindow::paintEvent(QPaintEvent *){
  QPixmap pm(size());
  QPainter p;
  p.begin(&pm);
  if (bkg_pixmap->isNull())
    pm.fill(backgroundColor());
  else
    p.drawTiledPixmap(0,0,width(),height(),*bkg_pixmap);
  int cy=5;
  srt[srt_title]->draw(&p,width()/2-srt[srt_title]->widthUsed()/2,cy,QRegion(0,0,width(),height()),colorGroup());
  cy+=srt[srt_title]->height();
  if ((pubkey->e<2)||(pubkey->n<2)){
    cy+=srt[srt_title]->height();
    srt[srt_brakkluczy]->draw(&p,width()/2-srt[srt_brakkluczy]->widthUsed()/2,cy,QRegion(0,0,width(),height()),colorGroup());
    cy+=srt[srt_brakkluczy]->height();
  }
  if ((pubkey->e>=2)&&(pubkey->n>=2)){
    srt[srt_kpub]->draw(&p,5,cy,QRegion(0,0,width(),height()),colorGroup());
    cy+=srt[srt_kpub]->height();
    srt[srt_tekstjawny]->draw(&p,5,cy,QRegion(0,0,width(),height()),colorGroup());
    srt[srt_tekstascii]->draw(&p,5,mln->y()+mln->height(),QRegion(0,0,width(),height()),colorGroup());
    srt[srt_tekstrsa]->draw(&p,5,tv[tv_ascii]->y()+tv[tv_ascii]->height(),QRegion(0,0,width(),height()),colorGroup());
  }
  p.end();
  bitBlt(this,0,0,&pm);
}

void EncodeWindow::resizeEvent(QResizeEvent *){
  for (int i=0; i<nr_srt; i++) srt[i]->setWidth(width());
  mln->setGeometry(0,srt[srt_title]->height()+srt[srt_kpub]->height()+srt[srt_tekstjawny]->height()+10,width(),3*srt[srt_tekstjawny]->height());
  tv[tv_ascii]->setGeometry(0,mln->y()+mln->height()+srt[srt_tekstascii]->height()+5,width(),mln->height());
  tv[tv_rsa]->setGeometry(0,tv[tv_ascii]->y()+tv[tv_ascii]->height()+srt[srt_tekstrsa]->height()+5,width(),height()-(tv[tv_ascii]->y()+tv[tv_ascii]->height()+srt[srt_tekstrsa]->height()+5));
}

QSimpleRichText * EncodeWindow::kpub2srt(){
  char es[10240],ns[10240];
  mpz_get_str(es,10,pubkey->e.get_mpz_t());
  mpz_get_str(ns,10,pubkey->n.get_mpz_t());
  return new QSimpleRichText(QString("<nobr>klucz publiczny <font color=blue>(e,n)=(")+QString(es)+QString(",")+QString(ns)+QString(")</font></nobr>"),font());
}

void EncodeWindow::updateTV(){
  QString s;
  for (int i=0; i<mln->numLines(); i++) s+=mln->textLine(i);
  QString s1;
  const char *t=s.latin1();
  tv[tv_ascii]->setText("");
  for (unsigned int i=0; i<strlen(t); i++){
    QString si;
    si.setNum(int(t[i]));
    QString ptva=tv[tv_ascii]->text();
    tv[tv_ascii]->setText(tv[tv_ascii]->text()+si+QString(", "));
    if (tv[tv_ascii]->visibleWidth()<tv[tv_ascii]->contentsWidth()){
      tv[tv_ascii]->setText(ptva+QString("<br>")+si+QString(", "));

    }
  }
  tv[tv_ascii]->scrollBy(0,tv[tv_ascii]->contentsHeight());
  msg->encode(t,*pubkey);
  tv[tv_rsa]->setText("");
  for (int i=0; i<msg->len; i++){
    char t1[10240];
    t1[0]=0;
    mpz_get_str(t1,10,msg->msg[i]);
    QString ptvr=tv[tv_rsa]->text();
    tv[tv_rsa]->setText(tv[tv_rsa]->text()+QString(t1)+QString(", "));
    if (tv[tv_rsa]->visibleWidth()<tv[tv_rsa]->contentsWidth()){
      tv[tv_rsa]->setText(ptvr+QString("<br>")+QString(t1)+QString(", "));
      
    }
  }
  tv[tv_rsa]->scrollBy(0,tv[tv_rsa]->contentsHeight());
  plain_text=mln->text();
  //  repaint(FALSE);
}
