#include <iostream.h>
#include <qpainter.h>
#include <qtextcodec.h>
#include "keygenw.H"

mpz_class KeyGenWindow::p=0;
mpz_class KeyGenWindow::q=0;

KeyGenWindow::KeyGenWindow(RSAKey *pub,RSAKey *priv,QPixmap *bkg,QWidget *parent,char *name) : QWidget(parent,name)
{
  // cout<<__PRETTY_FUNCTION__<<endl;
  pubkey=pub;
  privkey=priv;
  // p=0;q=0;n=0;fi=0;e=0;d=0;
  if (pubkey->n>0) n=pubkey->n;

  for (int i=0; i<nr_le; i++){
    le[i]=new QLineEdit(this);
    le[i]->show();
  }

  connect(le[le_p],SIGNAL(textChanged(const QString &)),this,SLOT(changeP(const QString &)));
  connect(le[le_q],SIGNAL(textChanged(const QString &)),this,SLOT(changeQ(const QString &)));
  connect(le[le_e],SIGNAL(textChanged(const QString &)),this,SLOT(changeE(const QString &)));

  QTextCodec *tc=QTextCodec::codecForName("iso8859-2");
  srt[srt_title]=new QSimpleRichText(tc->toUnicode("<h2><font color=darkblue>Generowanie pary kluczy RSA</font></h2>"),font());
  srt[srt_opispq]=new QSimpleRichText(tc->toUnicode("Do wygenerowania kluczy RSA potrzebne s dwie liczby pierwsze p i q. Wpisz poniej dwie dowolne liczby (program sam znajdzie najblisze im liczby pierwsze)"),font());
  srt[srt_ple]=new QSimpleRichText(tc->toUnicode("p0="),font());
  srt[srt_qle]=new QSimpleRichText(tc->toUnicode("q0="),font());
  srt[srt_p]=p2srt();
  srt[srt_q]=q2srt();
  srt[srt_opisn]=new QSimpleRichText(tc->toUnicode("Liczba n = p*q jest czci zarwno klucza publicznego jak i prywatnego"),font());
  srt[srt_n]=n2srt();
  srt[srt_nlc]=ilcyfr2srt(n);
  srt[srt_opisfi]=new QSimpleRichText(tc->toUnicode("Do obliczenia pozostaej cz kluczy potrzebna bdzie warto funkcji \"fi\" Eulera, fi(n)=(p-1)*(q-1)"),font());
  srt[srt_fi]=fi2srt();
  srt[srt_opise]=new QSimpleRichText(tc->toUnicode("Tzw. liczba szyfrujca e stanowi brakujc cz klucza publicznego. Moe ni by dowolna liczba mniejsza od fi(n) i wzgldnie pierwsza z fi(n). Wpisz dowoln liczb (program poszuka liczby e moliwie bliskiej tej wpisanej)."),font());
  srt[srt_ele]=new QSimpleRichText(tc->toUnicode("e0="),font());
  srt[srt_e]=e2srt();
  srt[srt_opisd]=new QSimpleRichText(tc->toUnicode("Tzw. liczba deszyfrujca d stanowi brakujc cz klucza prywatnego. Wyznacza si j ze wzoru <nobr>d=inv(e) mod fi(n)</nobr>"),font());
  srt[srt_d]=d2srt();
  srt[srt_opisklucze]=new QSimpleRichText(tc->toUnicode("No i mamy gotowe klucze:"),font());
  srt[srt_kpub]=kpub2srt();
  srt[srt_kpriv]=kpriv2srt();
  srt[srt_plc]=ilcyfr2srt(p);
  srt[srt_qlc]=ilcyfr2srt(q);
  //  setBackgroundPixmap(QPixmap("./marble.png"));
  //  bkg_pixmap.load("./marble.png");
  bkg_pixmap=bkg;
  e=pubkey->e;
  updateAll();
  //  cout<<__FUNCTION__<<endl;
}

KeyGenWindow::~KeyGenWindow(){
  for (int i=0; i<nr_le; i++) delete le[i];
  for (int i=0; i<nr_srt; i++) delete srt[i];
}

void KeyGenWindow::paintEvent(QPaintEvent *){
  for (int i=0; i<nr_le; i++) le[i]->repaint(FALSE);
  //  QPixmap pm(size());
  //  pm.fill(backgroundColor());
  //  QPixmap pm(*backgroundPixmap());
  
  QPixmap pm(size());
  QPainter p;
  p.begin(&pm);
  if (bkg_pixmap->isNull())
    pm.fill(backgroundColor());
  else 
    p.drawTiledPixmap(0,0,width(),height(),*bkg_pixmap);
  int cy=5;
  srt[srt_title]->draw(&p,width()/2-srt[srt_title]->widthUsed()/2,cy,QRegion(0,0,width(),height()),colorGroup());
  cy+=srt[srt_title]->height()+5;
  srt[srt_opispq]->draw(&p,5,cy,QRegion(0,0,width(),height()),colorGroup());
  srt[srt_ple]->draw(&p,5,le[le_p]->y()+3,QRegion(0,0,width(),height()),colorGroup());
  srt[srt_qle]->draw(&p,5,le[le_q]->y()+3,QRegion(0,0,width(),height()),colorGroup());
  srt[srt_p]->draw(&p,le[le_p]->x()+le[le_p]->width()+5,le[le_p]->y()+3,QRegion(0,0,width(),height()),colorGroup());
  srt[srt_plc]->draw(&p,le[le_p]->x()+le[le_p]->width()+5+srt[srt_p]->widthUsed()+5,le[le_p]->y()+3,QRegion(0,0,width(),height()),colorGroup());
  srt[srt_q]->draw(&p,le[le_q]->x()+le[le_q]->width()+5,le[le_q]->y()+3,QRegion(0,0,width(),height()),colorGroup());
  srt[srt_qlc]->draw(&p,le[le_q]->x()+le[le_q]->width()+5+srt[srt_q]->widthUsed()+5,le[le_q]->y()+3,QRegion(0,0,width(),height()),colorGroup());
  cy=le[le_q]->y()+le[le_q]->height()+10;
  srt[srt_opisn]->draw(&p,5,cy,QRegion(0,0,width(),height()),colorGroup());
  cy+=srt[srt_opisn]->height()+5;
  srt[srt_n]->draw(&p,5,cy,QRegion(0,0,width(),height()),colorGroup());
  srt[srt_nlc]->draw(&p,5+srt[srt_n]->widthUsed()+5,cy,QRegion(0,0,width(),height()),colorGroup());
  cy+=srt[srt_n]->height()+10;
  srt[srt_opisfi]->draw(&p,5,cy,QRegion(0,0,width(),height()),colorGroup());
  cy+=srt[srt_opisfi]->height()+5;
  srt[srt_fi]->draw(&p,5,cy,QRegion(0,0,width(),height()),colorGroup());
  cy+=srt[srt_fi]->height()+5;
  srt[srt_opise]->draw(&p,5,cy,QRegion(0,0,width(),height()),colorGroup());
  //  cy+=srt[srt_opise]->height()+5;
  srt[srt_ele]->draw(&p,5,le[le_e]->y()+3,QRegion(0,0,width(),height()),colorGroup());
  srt[srt_e]->draw(&p,le[le_e]->x()+le[le_e]->width()+5,le[le_e]->y()+3,QRegion(0,0,width(),height()),colorGroup());
  cy=le[le_e]->y()+le[le_e]->height()+10;
  srt[srt_opisd]->draw(&p,5,cy,QRegion(0,0,width(),height()),colorGroup());
  cy+=srt[srt_opisd]->height()+5;
  srt[srt_d]->draw(&p,5,cy,QRegion(0,0,width(),height()),colorGroup());
  cy+=srt[srt_d]->height()+15;
  srt[srt_opisklucze]->draw(&p,5,cy,QRegion(0,0,width(),height()),colorGroup());
  cy+=srt[srt_opisklucze]->height()+5;
  srt[srt_kpub]->draw(&p,5,cy,QRegion(0,0,width(),height()),colorGroup());
  cy+=srt[srt_kpub]->height()+5;
  srt[srt_kpriv]->draw(&p,5,cy,QRegion(0,0,width(),height()),colorGroup());
  p.end();
  bitBlt(this,0,0,&pm);
}

void KeyGenWindow::resizeEvent(QResizeEvent *){
  for (int i=0; i<nr_srt; i++) srt[i]->setWidth(width());
  le[le_p]->setGeometry(srt[srt_ple]->widthUsed()+5+5,(srt[srt_title]->height()+10+srt[srt_opispq]->height()+10),width()/2-(srt[srt_ple]->widthUsed())-10,srt[srt_ple]->height()+10);
  le[le_q]->setGeometry(le[le_p]->x(),le[le_p]->y()+le[le_p]->height(),le[le_p]->width(),le[le_p]->height());
  le[le_e]->setGeometry(le[le_q]->x(),le[le_q]->y()+le[le_p]->height()+10+srt[srt_opisn]->height()+5+srt[srt_n]->height()+10+srt[srt_opisfi]->height()+5+srt[srt_fi]->height()+5+srt[srt_opise]->height()+5,le[le_q]->width(),le[le_q]->height());
}

void KeyGenWindow::changeP(const QString &s){
  p=nextPrime(mpz_class(le[le_p]->text().latin1()));
  delete srt[srt_p]; srt[srt_p]=p2srt();
  delete srt[srt_plc]; srt[srt_plc]=ilcyfr2srt(p);
  updateNFiED();
  repaint(FALSE);
}

void KeyGenWindow::changeQ(const QString &s){
  q=nextPrime(mpz_class(le[le_q]->text().latin1()));
  if (p==q) q=nextPrime(p);
  delete srt[srt_q]; srt[srt_q]=q2srt();
  delete srt[srt_qlc]; srt[srt_qlc]=ilcyfr2srt(q);
  updateNFiED();
  repaint(FALSE);
}

void KeyGenWindow::changeE(const QString &s){
  if (fi>=4){
    e=mpz_class(le[le_e]->text().latin1());
    if (e>=fi) {e=0; return;}
    while (gcd(e,fi)!=1){
      e++;
      if (e>=fi) e=2;
    }
    delete srt[srt_e]; srt[srt_e]=e2srt();
    updateD();
    repaint(FALSE);
  }
}

void KeyGenWindow::updateNFiED(){
  n=p*q;
  delete srt[srt_n]; srt[srt_n]=n2srt();
  delete srt[srt_nlc]; srt[srt_nlc]=ilcyfr2srt(n);
  fi=(p-1)*(q-1);
  if (fi<0) fi=0;
  delete srt[srt_fi]; srt[srt_fi]=fi2srt();
  if (fi>=4) changeE(le[le_e]->text());
}

void KeyGenWindow::updateD(){
  d=invert(e,fi);
  delete srt[srt_d]; srt[srt_d]=d2srt();
  if ((e>0)&&(d>0)&&(n>0)) updateKeys();
}

void KeyGenWindow::updateAll(){
  delete srt[srt_p]; srt[srt_p]=p2srt();
  delete srt[srt_q]; srt[srt_q]=q2srt();
  n=p*q;
  delete srt[srt_n]; srt[srt_n]=n2srt();
  //  delete srt[srt_nlc]; srt[srt_nlc]=ilcyfr2srt(pubkey->n);
  fi=(p-1)*(q-1);
  if (fi<0) fi=0;
  delete srt[srt_fi]; srt[srt_fi]=fi2srt();
  delete srt[srt_e]; srt[srt_e]=e2srt();
  d=invert(e,fi);
  delete srt[srt_d]; srt[srt_d]=d2srt();
  if ((e>0)&&(d>0)&&(n>0)) updateKeys();
}

void KeyGenWindow::updateKeys(){
  delete srt[srt_kpub]; srt[srt_kpub]=kpub2srt();
  delete srt[srt_kpriv]; srt[srt_kpriv]=kpriv2srt();
  delete srt[srt_plc]; srt[srt_plc]=ilcyfr2srt(p);
  delete srt[srt_qlc]; srt[srt_qlc]=ilcyfr2srt(q);
  pubkey->n=n;
  pubkey->e=e;
  privkey->n=n;
  privkey->d=d;
}

QSimpleRichText * KeyGenWindow::p2srt(){
  char buff[10240];
  mpz_get_str(buff,10,p.get_mpz_t());
  QString col;
  if (p>=2) col=QString("blue"); else col=QString("red");
  return new QSimpleRichText(QString("<font color=")+col+QString(">p=")+QString(buff)+QString("</font>"),font());
}

QSimpleRichText * KeyGenWindow::q2srt(){
  char buff[10240];
  mpz_get_str(buff,10,q.get_mpz_t());
  QString col;
  if (q>=2) col=QString("blue"); else col=QString("red");
  return new QSimpleRichText(QString("<font color=")+col+QString(">q=")+QString(buff)+QString("</font>"),font());
}

QSimpleRichText * KeyGenWindow::n2srt(){
  char buff[10240];
  mpz_get_str(buff,10,n.get_mpz_t());
  QString col;
  if (n>=4) col=QString("blue"); else col=QString("red");
  return new QSimpleRichText(QString("<font color=")+col+QString(">n=")+QString(buff)+QString("</font>"),font());
}

QSimpleRichText * KeyGenWindow::fi2srt(){
  char buff[10240];
  mpz_get_str(buff,10,fi.get_mpz_t());
  QString col;
  if (fi>=2) col=QString("blue"); else col=QString("red");
  return new QSimpleRichText(QString("<font color=")+col+QString(">fi(n)=")+QString(buff)+QString("</font>"),font());
}

QSimpleRichText * KeyGenWindow::e2srt(){
  char buff[10240];
  mpz_get_str(buff,10,e.get_mpz_t());
  QString col;
  if (e>=2) col=QString("blue"); else col=QString("red");
  return new QSimpleRichText(QString("<font color=")+col+QString(">e=")+QString(buff)+QString("</font>"),font());
}

QSimpleRichText * KeyGenWindow::d2srt(){
  char buff[10240];
  mpz_get_str(buff,10,d.get_mpz_t());
  QString col;
  if (d>=2) col=QString("blue"); else col=QString("red");
  return new QSimpleRichText(QString("<font color=")+col+QString(">d=")+QString(buff)+QString("</font>"),font());
}

QSimpleRichText * KeyGenWindow::kpub2srt(){
  char es[10240],ns[10240];
  mpz_get_str(es,10,e.get_mpz_t());
  mpz_get_str(ns,10,n.get_mpz_t());
  QString col;
  if ((n>=2)&&(e>=2)&&(d>=2)) col=QString("blue"); else col=QString("red");
  return new QSimpleRichText(QString("<nobr>klucz publiczny <font color=")+col+QString(">(e,n)=(")+QString(es)+QString(",")+QString(ns)+QString(")</font></nobr>"),font());
}

QSimpleRichText * KeyGenWindow::kpriv2srt(){
  char ds[10240],ns[10240];
  mpz_get_str(ds,10,d.get_mpz_t());
  mpz_get_str(ns,10,n.get_mpz_t());
  QString col;
  if ((n>=2)&&(e>=2)&&(d>=2)) col=QString("blue"); else col=QString("red");
  return new QSimpleRichText(QString("<nobr>klucz prywatny <font color=")+col+QString(">(d,n)=(")+QString(ds)+QString(",")+QString(ns)+QString(")</font></nobr>"),font());
}

QSimpleRichText * KeyGenWindow::ilcyfr2srt(mpz_class z){
  char zs[10240];
  mpz_get_str(zs,10,z.get_mpz_t());
  QString zs_lc;
  zs_lc.setNum(strlen(zs));
  if (z<=0) return new QSimpleRichText(QString(""),font());
  if (strlen(zs)<=0) return new QSimpleRichText(QString(""),font());
  if (strlen(zs)==1) return new QSimpleRichText(QString("<nobr>(")+zs_lc+QString(" cyfra)</nobr>"),font());
  if ((strlen(zs)>=12)&&(strlen(zs)<=14)) return new QSimpleRichText(QString("<nobr>(")+zs_lc+QString(" cyfr)</nobr>"),font()); 
  if ((strlen(zs)%10>=2)&&(strlen(zs)%10<=4)) return new QSimpleRichText(QString("<nobr>(")+zs_lc+QString(" cyfry)</nobr>"),font());
  return new QSimpleRichText(QString("<nobr>(")+zs_lc+QString(" cyfr)</nobr>"),font());
}

