#include "rsa.H"

RSAKey::RSAKey(){
  n=0;
  d=0;
  e=0;
}

RSAEncodedMsg::RSAEncodedMsg(){
  msg=NULL;
  len=0;
}

RSAEncodedMsg::~RSAEncodedMsg(){
  if (msg!=NULL) {
    delete[] msg;
  }
}

RSAEncodedMsg::RSAEncodedMsg(const RSAEncodedMsg &m){
  if (m.len>0) {
    msg=new mpz_t[m.len];
    for (int i=0; i<m.len; i++) mpz_init(msg[i]);
  } else msg=NULL;
  len=m.len;
}

RSAEncodedMsg & RSAEncodedMsg::operator=(const RSAEncodedMsg &m){
  if (&m!=this){
    if (m.len>0) {
      msg=new mpz_t[m.len];
      for (int i=0; i<m.len; i++) mpz_init(msg[i]);
    } else msg=NULL;
    len=m.len;
  }
  return *this;
}

void RSAEncodedMsg::setLen(int n){
  if (len>0) delete[] msg;
  msg=new mpz_t[n];
  for (int i=0; i<n; i++) mpz_init(msg[i]);
  len=n;
}

mpz_class powModN(mpz_class a,mpz_class x,mpz_class n){
  mpz_t p;
  mpz_init(p);
  mpz_powm(p,a.get_mpz_t(),x.get_mpz_t(),n.get_mpz_t());
  mpz_class w(p);
  mpz_clear(p);
  return w; 
}

mpz_class invert(mpz_class a,mpz_class n){
  mpz_t i;
  mpz_init(i);
  mpz_invert(i,a.get_mpz_t(),n.get_mpz_t());
  mpz_class w(i);
  mpz_clear(i);
  return w;
}

mpz_class gcd(mpz_class a,mpz_class n){
  mpz_t g;
  mpz_init(g);
  mpz_gcd(g,a.get_mpz_t(),n.get_mpz_t());
  mpz_class w(g);
  mpz_clear(g);
  return w;
}

mpz_class nextPrime(mpz_class z){
  mpz_t np;
  mpz_init(np);
  mpz_nextprime(np,z.get_mpz_t());
  mpz_class w(np);
  mpz_clear(np);
  return w;
}

void RSAEncodedMsg::encode(const char *t,RSAKey pubkey){
  setLen(strlen(t));
  for (int i=0; i<len; i++){
    mpz_class a=int(t[i]);
    mpz_class w=powModN(a,pubkey.e,pubkey.n);
    mpz_set(msg[i],w.get_mpz_t());
  }
}

void RSAEncodedMsg::decode(char *t,RSAKey privkey){
  for (int i=0; i<len; i++){
    //    mpz_class a=int(t[i]);
    mpz_class w=powModN(mpz_class(msg[i]),privkey.d,privkey.n);
    //    mpz_set(msg[i],w.get_mpz_t());
    t[i]=char(w.get_si());
  }
}
